require 'test/unit'
require 'test/unit/testsuite'
require 'test/unit/ui/console/testrunner'
require 'amrita2/rd'
require 'rexml/document'

module Amrita2RDTest
  include Amrita2::HtmlTag
  def assert_equal_x(a, s)
    s = s.gsub(/(\>)\s*(<)/) {  $1.to_s+$2.to_s }.strip
    assert_equal(a, s)
  end


  class TestRD < Test::Unit::TestCase
    include Amrita2
    include Amrita2::RDTmpl
    include Amrita2RDTest

    def test_sanitize
      s = "\na<b>c\n"
      ret = RDAProcessor.new.process(s)
      assert_equal_x("<html><body><p>a&lt;b&gt;c\n</p></body></html>",ret)
    end

    def test_simple
      s = 'a'
      ret = RDAProcessor.new.process(s)
      assert_equal_x("<html><body><p>a\n</p></body></html>",ret)
    end

    def test_em
      s = '((*a*))'
      ret = RDAProcessor.new.process(s)
      assert_equal_x("<html><body><p><em>*a*</em></p></body></html>",ret)
    end

    def test_inline
      s = '(({code}))((|var|))((%key%))'
      ret = RDAProcessor.new.process(s)
      #assert_equal_x("<html><body><p><code>+code+</code><var>_var_</var><kbd>|key|</kbd></p></body></html>",ret)
      assert_equal_x("<html><body><p><code>code</code><var>var</var><kbd>key</kbd></p></body></html>",ret)
    end

    def test_url
      s = '((<ruby home|URL:http://www.ruby-lang.org>))'
      ret = RDAProcessor.new.process(s)
      assert_equal_x("<html><body><p><a href='http://www.ruby-lang.org'>ruby home</a></p></body></html>",ret)
    end

    def test_url2
      s = '((<rubytalk:12345>))'
      ret = RDAProcessor.new.process(s)
      assert_equal_x("<html><body><p><a href='http://blade.nagaokaut.ac.jp/cgi-bin/scat.rb/ruby/ruby-talk/12345'>[ruby-talk:12345]</a></p></body></html>",ret)
    end

    def test_url3
      s = '((<raa:amrita2:template library>))'
      ret = RDAProcessor.new.process(s)
      assert_equal_x("<html><body><p><a href='http://raa.ruby-lang.org/list.rhtml?name=amrita2'>[template library]</a></p></body></html>",ret)
    end

    def test_table
      s = <<-END
--- table table1 4
      hhh hhh  hhhhh hh
      111 2222 3333   4444
      aaa bbb cccc dddd
END
      ret = RDAProcessor.new.process(s)
      assert_equal_x("<html><body><table border='1'><tr><th>hhh</th><th>hhh</th><th>hhhhh</th><th>hh</th></tr><tr class='a'><th>111</th><td>2222</td><td>3333</td><td>4444</td></tr><tr class='b'><th>aaa</th><td>bbb</td><td>cccc</td><td>dddd</td></tr></table></body></html>", ret)
    end

    def test_blockquote
      s = <<-END
--- blockquote http://www.ruby-lang.org/
    aaaa

bbbb
END
      ret = RDAProcessor.new.process(s)
      assert_equal_x("<html><body><blockquote cite='http://www.ruby-lang.org/'><p>aaaa </p><p>(<a href='http://www.ruby-lang.org/'>http://www.ruby-lang.org/</a>)</p></blockquote><p>bbbb\n</p></body></html>", ret)
    end

    def test_list
      s = <<-END
* 1
* 2
  (1) a
      aaa
  (2) b
  zzz
END
      ret = RDAProcessor.new.process(s)
      assert_equal_x("<html><body><ul><li>1 </li><li><p>2\n</p><ol><li>a       aaa </li><li>b </li></ol><p>zzz\n</p></li></ul></body></html>", ret)
    end

    def test_list2
      s = <<-END
* ((<a|URL:./a>))
* ((<b|URL:./b>))
END
      ret = RDAProcessor.new.process(s)
      assert_equal_x("<html><body><ul><li><a href='./a'>a</a></li><li><a href='./b'>b</a></li></ul></body></html>", ret)
    end

    def test_desclist
      s = <<-END
:a
   aaaa
:b
   bbbb
   bbbb
END
      ret = RDAProcessor.new.process(s)
      assert_equal_x("<html><body><dl><dt>a</dt><dd>aaaa </dd><dt>b</dt><dd>bbbb bbbb </dd></dl></body></html>", ret)
    end

    def test_head
      s = '== title ttttt'
      ret = RDAProcessor.new.process(s)
      assert_equal_x("<html><body><h2>title ttttt</h2></body></html>",ret)
    end

    def test_ext
      parts_spec = Amrita2::define_template_spec do 
        dynamic_element(:xxx) do
          parts_element(:parts1, :delete_id=>true, :use_args=>true)
          parts_element(:parts2, :delete_id=>true)
        end
      end

      doc = REXML::Document.new  "<zz id='xxx'><a id='parts1' href='xxxx/$2'>$1</a><span id='parts2' class='yyyy'>111</span></zz>"
      parts = parts_spec.create_template(doc)
      
      
      s = '((<parts1:12345:abcde>))xx((<parts2:98765>))'
      ret = RDAProcessor.new(:extention_processor=>parts).process(s)
      assert_equal_x("<html><body><p><a href='xxxx/abcde'>12345</a>xx<span class='yyyy'>98765</span></p></body></html>", ret)
    end

    def test_document
      s = <<-END
= header1
== header12
= header2
 
* ((<hello sample|URL:./hello>))
* ((<hello sample source|URL:./hello.rb>))

== header22

aaa
bbb

* ccc((*eeee*))

END
      ret = RDAProcessor.new.process(s)
      assert_equal_x("<html><body><h1>header1</h1><h2>header12</h2><h1>header2</h1><ul><li><a href='./hello'>hello sample</a></li><li><a href='./hello.rb'>hello sample source</a></li></ul><h2>header22</h2><p>aaa\nbbb\n</p><ul><li>ccc<em>*eeee*</em></li></ul></body></html>", ret)
    end      

    def test_head
      s = 'a'
      ret = RDAProcessor.new.process(s, :title=>"ttt", :css=>"default.css")
      assert_equal_x("<html><head><title>ttt</title><link href='default.css' rel='stylesheet' type='text/css'></link></head><body><p>a\n</p></body></html>",ret)
    end
  end
end

if __FILE__ == $0
  include Amrita2RDTest
  cname = ARGV.shift
  if cname
    method = ARGV.shift
    if method
      eval "Test::Unit::UI::Console::TestRunner.run(#{cname}.new(\"#{method}\"))"
    else
      cls = eval(cname)
      Test::Unit::UI::Console::TestRunner.run(cls)
    end
  else
    suite = Test::Unit::TestSuite.new
    suite << TestRD.suite
    Test::Unit::UI::Console::TestRunner.run(suite)
  end
end

# ruby -I~/amrita2/lib ~/amrita2/test/test_rd.rb TestRD test_table |ruby -pe '$_.sub!(/^ */, "") ' 
